@@ -4,8 +4,9 @@ from __future__ import absolute_import, print_function
 import datetime as dt
 import json
 import copy
+import uuid
 
-from marshmallow import base, exceptions, fields, utils
+from marshmallow import base, fields, utils
 from marshmallow.compat import (with_metaclass, iteritems, text_type,
                                 binary_type, OrderedDict)
 
@@ -45,8 +46,13 @@ class SerializerOpts(object):
 
     def __init__(self, meta):
         self.fields = getattr(meta, 'fields', ())
+        self.additional = getattr(meta, 'additional', ())
+        if self.fields and self.additional:
+            raise ValueError("Cannot set both `fields` and `additional` options"
+                            " for the same serializer.")
         self.exclude = getattr(meta, 'exclude', ())
         self.strict = getattr(meta, 'strict', False)
+        self.dateformat = getattr(meta, 'dateformat', None)
 
 
 class BaseSerializer(base.SerializerABC):
@@ -62,6 +68,10 @@ class BaseSerializer(base.SerializerABC):
         list: fields.Raw,
         set: fields.Raw,
         int: fields.Integer,
+        uuid.UUID: fields.UUID,
+        dt.time: fields.Time,
+        dt.date: fields.Date,
+        dt.timedelta: fields.TimeDelta,
     }
 
     class Meta(object):
@@ -72,7 +82,18 @@ class BaseSerializer(base.SerializerABC):
             class Meta:
                 fields = ("id", "email", "date_created")
                 exclude = ("password", "secret_attribute")
-                strict = False
+
+        Available options:
+
+        - ``fields``: Tuple or list of fields to include in the serialized result.
+        - ``additional``: Tuple or list of fields to include *in addition* to the
+            explicitly declared fields. ``additional`` and ``fields`` are
+            mutually-exclusive options.
+        - ``exclude``: Tuple or list of fields to exclude in the serialized result.
+        - ``dateformat``: Date format for all DateTime fields that do not have their
+            date format explicitly specified.
+        - ``strict``: If ``True``, raise errors during marshalling rather than
+            storing them.
         '''
         pass
 
@@ -84,12 +105,17 @@ class BaseSerializer(base.SerializerABC):
         self.exclude = exclude or ()
         self.prefix = prefix
         self.fields = self.__get_fields()  # Dict of fields
-        self.errors = {}
-        self.strict = strict
-        #: The serialized data as an ``OrderedDict``
-        self.data = self.to_data()
+        if strict or self.opts.strict:
+            self.strict = True
+        else:
+            self.strict = False
+        #: Callable marshalling object
+        self._marshal = fields.Marshaller(prefix=self.prefix, strict=self.strict)
+        # Store the marshalled data
+        # MUST occur upon initialization
+        self._data = self.marshal(self.obj, self.fields)
         if extra:
-            self.data.update(extra)
+            self._data.update(extra)
 
     def __get_fields(self):
         '''Return the declared fields for the object as an OrderedDict.'''
@@ -100,9 +126,13 @@ class BaseSerializer(base.SerializerABC):
         for field_name, field_obj in iteritems(declared_fields):
             ret[field_name] = field_obj
 
-        # If "fields" option is specified, use those fields
         if self.opts.fields:
-            ret = self.__get_opts_fields(ret)
+            # Return only fields specified in fields option
+            ret = self.__get_opts_fields(ret, self.opts.fields)
+        elif self.opts.additional:
+            # Return declared fields + additional fields
+            field_names = tuple(ret.keys()) + tuple(self.opts.additional)
+            ret = self.__get_opts_fields(ret, field_names)
 
         # if only __init__ param is specified, only return those fields
         if self.only:
@@ -113,7 +143,7 @@ class BaseSerializer(base.SerializerABC):
                         '"{0}" is not a valid field for {1}.'
                             .format(field_name, self.obj))
                 filtered[field_name] = ret[field_name]
-            self.__set_parents(filtered)
+            self.__initialize_fields(filtered)
             return filtered
 
         # If "exclude" option or param is specified, remove those fields
@@ -125,19 +155,29 @@ class BaseSerializer(base.SerializerABC):
             for field_name in excludes:
                 ret.pop(field_name, None)
         # Set parents
-        self.__set_parents(ret)
+        self.__initialize_fields(ret)
         return ret
 
-    def __set_parents(self, fields_dict):
-        '''Set the parents of all field objects in fields_dict to self.'''
+    def __initialize_fields(self, fields_dict):
+        '''Set the parents of all field objects in fields_dict to self, and
+        set the dateformat specified in ``class Meta``, if necessary.
+        '''
         for _, field_obj in iteritems(fields_dict):
             if not field_obj.parent:
                 field_obj.parent = self
+            if isinstance(field_obj, fields.DateTime):
+                if field_obj.dateformat is None:
+                    field_obj.dateformat = self.opts.dateformat
         return fields_dict
 
-    def __get_opts_fields(self, declared_fields):
-        '''Return only those field_name:field_obj pairs specified in the fields
-        option of class Meta.
+    def __get_opts_fields(self, declared_fields, field_names):
+        '''Return only those field_name:field_obj pairs specified by
+        ``field_names``.
+
+        :param dict declared_fields: The original dictionary of explicitly
+            declared fields.
+        :param tuple field_names: List of field names to include in the final
+            return dictionary.
         '''
         # Convert obj to a dict
         if not isinstance(self.opts.fields, (list, tuple)):
@@ -150,25 +190,23 @@ class BaseSerializer(base.SerializerABC):
                 return declared_fields
         else:
             obj_dict = obj_marshallable
-        new = OrderedDict()
-        for key in self.opts.fields:
+        ret = OrderedDict()
+        for key in field_names:
             if key in declared_fields:
-                new[key] = declared_fields[key]
+                ret[key] = declared_fields[key]
             else:
                 try:
-                    attribute_type = type(obj_dict[key])
+                    if obj_dict:
+                        attribute_type = type(obj_dict[key])
+                        field_obj = self.TYPE_MAPPING.get(attribute_type, fields.Raw)()
+                    else:  # Object is None
+                        field_obj = fields.Raw()
                 except KeyError:
                     raise AttributeError(
                         '"{0}" is not a valid field for {1}.'.format(key, self.obj))
                 # map key -> field (default to Raw)
-                new[key] = self.TYPE_MAPPING.get(attribute_type, fields.Raw)()
-        return new
-
-
-    @property
-    def json(self):
-        '''The data as a JSON string.'''
-        return self.to_json()
+                ret[key] = field_obj
+        return ret
 
     def marshal(self, data, fields_dict):
         """Takes the data (a dict, list, or object) and a dict of fields.
@@ -178,39 +216,35 @@ class BaseSerializer(base.SerializerABC):
         :param dict fields_dict: A dict whose keys will make up the final serialized
                        response output
         """
-        if utils.is_collection(data):
-            return [self.marshal(d, fields_dict) for d in data]
-        items = []
-        for attr_name, field_obj in iteritems(fields_dict):
-            key = self.prefix + attr_name
-            try:
-                if isinstance(field_obj, dict):
-                    item = (key, self.marshal(data, field_obj))
-                else:
-                    try:
-                        item = (key, field_obj.output(attr_name, data))
-                    except TypeError:
-                        # field declared as a class, not an instance
-                        if issubclass(field_obj, base.FieldABC):
-                            msg = ('Field for "{0}" must be declared as a '
-                                            "Field instance, not a class. "
-                                            'Did you mean "fields.{1}()"?'
-                                            .format(attr_name, field_obj.__name__))
-                            raise TypeError(msg)
-                        raise
-            except exceptions.MarshallingError as err:  # Store errors
-                if self.strict or self.opts.strict:
-                    raise err
-                self.errors[key] = text_type(err)
-                item = (key, None)
-            items.append(item)
-        return OrderedDict(items)
+        return self._marshal(data, fields_dict)
 
-    def to_data(self, *args, **kwargs):
-        return self.marshal(self.obj, self.fields, *args, **kwargs)
+    @property
+    def data(self):
+        '''The serialized data as an ``OrderedDict``.
+        '''
+        return self._data
+
+    @property
+    def json(self):
+        '''The data as a JSON string.'''
+        return self.to_json()
+
+    @property
+    def errors(self):
+        '''Dictionary of errors raised during serialization.'''
+        return self._marshal.errors
 
     def to_json(self, *args, **kwargs):
-        return json.dumps(self.data, *args, **kwargs)
+        '''Return the JSON representation of the data. Takes the same arguments
+        as Pythons built-in ``json.dumps``.
+        '''
+        ret = json.dumps(self.data, *args, **kwargs)
+        # On Python 2, json.dumps returns bytestrings
+        # On Python 3, json.dumps returns unicode
+        # Ensure that a bytestring is returned
+        if isinstance(ret, text_type):
+            return binary_type(ret.encode('utf-8'))
+        return ret
 
     def is_valid(self, field_names=None):
         """Return ``True`` if all data are valid, ``False`` otherwise.
@@ -233,7 +267,8 @@ class Serializer(with_metaclass(SerializerMeta, BaseSerializer)):
     '''Base serializer class with which to define custom serializers.
 
     Example usage:
-    ::
+
+    .. code-block:: python
 
         from datetime import datetime
         from marshmallow import Serializer, fields
@@ -256,7 +291,7 @@ class Serializer(with_metaclass(SerializerMeta, BaseSerializer)):
         serialized = PersonSerializer(person)
         serialized.data
         # OrderedDict([('name', u'Guido van Rossum'),
-                        ('date_born', 'Sat, 09 Nov 2013 00:10:29 -0000')])
+        #                ('date_born', 'Sat, 09 Nov 2013 00:10:29 -0000')])
 
     :param obj: The object or collection of objects to be serialized.
     :param dict extra: A dict of extra attributes to bind to the serialized result.
